#include <MellowPlayer/Infrastructure/BuildConfig.hpp>

using namespace MellowPlayer::Infrastructure;

QString BuildConfig::getVersion()
{
    return "3.6.8.0";
}

int BuildConfig::getVersionMajor()
{
    return 3;
}

int BuildConfig::getVersionMinor()
{
    return 6;
}

int BuildConfig::getVersionPatch()
{
    return 8;
}

int BuildConfig::getVersionBuildNumber()
{
    return 0;
}

QString BuildConfig::getDefaultTheme()
{
    return "Adaptive";
}

QString BuildConfig::getBuildDate()
{
    return "2023-01-19";
}

QString BuildConfig::getSourceDir()
{
    return "/builddir/build/BUILD/MellowPlayer-3.6.8";
}

static QString compilerString()
{
#if defined(Q_CC_CLANG) // must be before GNU, because clang claims to be GNU too
    QString isAppleString;
#if defined(__apple_build_version__) // Apple clang has other version numbers
    isAppleString = QLatin1String(" (Apple)");
#endif
    return QLatin1String("Clang ") + QString::number(__clang_major__) + QLatin1Char('.') + QString::number(__clang_minor__) + isAppleString;
#elif defined(Q_CC_GNU)
    return QLatin1String("GCC ") + QLatin1String(__VERSION__);
#elif defined(Q_CC_MSVC)
    if (_MSC_VER >= 1800) // 1800: MSVC 2013 (yearly release cycle)
        return QLatin1String("MSVC ") + QString::number(2008 + ((_MSC_VER / 100) - 13));
    if (_MSC_VER >= 1500) // 1500: MSVC 2008, 1600: MSVC 2010, ... (2-year release cycle)
        return QLatin1String("MSVC ") + QString::number(2008 + 2 * ((_MSC_VER / 100) - 15));
#else
    return QObject::tr("<unknown compiler>");
#endif
}

QString BuildConfig::buildInfo()
{
    return BuildConfigStrings().builtOnStr().arg(
            QString(__DATE__),
            QString(__TIME__),
            compilerString(),
            QString::number(QSysInfo::WordSize),
            QString(QT_VERSION_STR));
}
